@@ -1,5 +1,5 @@
 /*  XMMS STATUS PLUGIN - Status Docklet Plugin for XMMS
- *  Copyright (C) 2000  Ian Campbell.
+ *  Copyright (C) 2000,2001,2002  Ian Campbell.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public Licensse as published by
@@ -25,6 +25,14 @@
 
 #include "xmms-status-plugin.h"
 
+#define STATUS_DOCKLET_IMAGE_DELAY_MIN 0.0
+#define STATUS_DOCKLET_IMAGE_DELAY_MAX 5000.0
+
+#define STATUS_DOCKLET_BALLOON_DELAY_MIN 0.0
+#define STATUS_DOCKLET_BALLOON_DELAY_MAX 10.0
+
+#define STATUS_DOCKLET_CONFIG_SECTION "status_docklet"
+
 StatusDockletConfig status_docklet_config;
 
 static void status_docklet_cancel_cb(GtkWidget *widget, gpointer data);
@@ -45,6 +53,10 @@ static GtkObject *playing_image_adjustment;
 static GtkObject *paused_image_adjustment;
 static GtkObject *stopped_image_adjustment;
 
+static GtkObject *balloon_delay_adjustment;
+
+static GtkWidget *freedesktop_support_check_button;
+
 static GtkWidget *config_window;
 
 static gchar *button_action_text[] = {
@@ -99,27 +111,45 @@ void status_docklet_save_config(void)
 	status_docklet_config.paused_image_delay = (gint)GTK_ADJUSTMENT(paused_image_adjustment)->value;
 	status_docklet_config.stopped_image_delay = (gint)GTK_ADJUSTMENT(stopped_image_adjustment)->value;
 
+	status_docklet_config.balloon_delay = (gint)(GTK_ADJUSTMENT(balloon_delay_adjustment)->value);
+
 	for (mod=0; mod<STATUS_DOCKLET_BUTTON_MODIFIER_LAST; mod++) {
 		for (button=0; button<STATUS_DOCKLET_BUTTON_LAST; button++) {
 			key = g_strdup_printf("button_action_%s%s%d",
 					      (mod==STATUS_DOCKLET_BUTTON_MODIFIER_NONE) ? "" : button_modifier_text[mod],
 					      (mod==STATUS_DOCKLET_BUTTON_MODIFIER_NONE) ? "" : "_",
 					      button+1);
-			xmms_cfg_write_int(cfgfile, "status_docklet", key, status_docklet_config.button_actions[mod][button]);
+			xmms_cfg_write_int(cfgfile, STATUS_DOCKLET_CONFIG_SECTION, 
+					   key, status_docklet_config.button_actions[mod][button]);
 			g_free(key);
 		}
 	}
 	
-	if ( status_docklet_config.playing_image )
-		xmms_cfg_write_string(cfgfile, "status_docklet", "playing_image", status_docklet_config.playing_image);
-	if ( status_docklet_config.paused_image )
-		xmms_cfg_write_string(cfgfile, "status_docklet", "paused_image", status_docklet_config.paused_image);
-	if ( status_docklet_config.stopped_image )
-		xmms_cfg_write_string(cfgfile, "status_docklet", "stopped_image", status_docklet_config.stopped_image);
+	status_docklet_config.freedesktop_docklet_support = gtk_toggle_button_get_active
+		(GTK_TOGGLE_BUTTON(freedesktop_support_check_button));
 
-	xmms_cfg_write_int(cfgfile, "status_docklet", "playing_image_delay", status_docklet_config.playing_image_delay);
-	xmms_cfg_write_int(cfgfile, "status_docklet", "paused_image_delay", status_docklet_config.paused_image_delay);
-	xmms_cfg_write_int(cfgfile, "status_docklet", "stopped_image_delay", status_docklet_config.stopped_image_delay);
+	if ( status_docklet_config.playing_image )
+		xmms_cfg_write_string(cfgfile, STATUS_DOCKLET_CONFIG_SECTION, 
+				      "playing_image", status_docklet_config.playing_image);
+	if ( status_docklet_config.paused_image )
+		xmms_cfg_write_string(cfgfile, STATUS_DOCKLET_CONFIG_SECTION, 
+				      "paused_image", status_docklet_config.paused_image);
+	if ( status_docklet_config.stopped_image )
+		xmms_cfg_write_string(cfgfile, STATUS_DOCKLET_CONFIG_SECTION, 
+				      "stopped_image", status_docklet_config.stopped_image);
+
+	xmms_cfg_write_int(cfgfile, STATUS_DOCKLET_CONFIG_SECTION, 
+			   "playing_image_delay", status_docklet_config.playing_image_delay);
+	xmms_cfg_write_int(cfgfile, STATUS_DOCKLET_CONFIG_SECTION, 
+			   "paused_image_delay", status_docklet_config.paused_image_delay);
+	xmms_cfg_write_int(cfgfile, STATUS_DOCKLET_CONFIG_SECTION, 
+			   "stopped_image_delay", status_docklet_config.stopped_image_delay);
+
+	xmms_cfg_write_int(cfgfile, STATUS_DOCKLET_CONFIG_SECTION,
+			   "balloon_delay", status_docklet_config.balloon_delay);
+
+	xmms_cfg_write_boolean(cfgfile, STATUS_DOCKLET_CONFIG_SECTION, 
+			       "freedesktop_system_tray", status_docklet_config.freedesktop_docklet_support);
 
 	xmms_cfg_write_default_file(cfgfile);
 	
@@ -133,24 +163,35 @@ void status_docklet_load_config(void)
 	ConfigFile *cfgfile;
 	gchar *tmp;
 	gint mod, button;
+	gint delay;
 
 	/* Setup defaults */
 	status_docklet_config.button_actions[STATUS_DOCKLET_BUTTON_MODIFIER_NONE][0] = STATUS_DOCKLET_BUTTON_ACTION_TOGGLE_WINDOWS;
 	status_docklet_config.button_actions[STATUS_DOCKLET_BUTTON_MODIFIER_NONE][1] = STATUS_DOCKLET_BUTTON_ACTION_FORWARD;
 	status_docklet_config.button_actions[STATUS_DOCKLET_BUTTON_MODIFIER_NONE][2] = STATUS_DOCKLET_BUTTON_ACTION_MENU;
-	status_docklet_config.button_actions[STATUS_DOCKLET_BUTTON_MODIFIER_NONE][3] = STATUS_DOCKLET_BUTTON_ACTION_NONE;
-	status_docklet_config.button_actions[STATUS_DOCKLET_BUTTON_MODIFIER_NONE][4] = STATUS_DOCKLET_BUTTON_ACTION_NONE;
+	/* and the rest ... */
+	for(button = 3; button < STATUS_DOCKLET_BUTTON_LAST; button++)
+		status_docklet_config.button_actions
+			[STATUS_DOCKLET_BUTTON_MODIFIER_NONE][button] = 
+			STATUS_DOCKLET_BUTTON_ACTION_NONE;
+
 	for (mod=1; mod<STATUS_DOCKLET_BUTTON_MODIFIER_LAST; mod++)
 		for (button=0; button<STATUS_DOCKLET_BUTTON_LAST; button++)
-			status_docklet_config.button_actions[mod][button] = STATUS_DOCKLET_BUTTON_ACTION_NONE;
+			status_docklet_config.button_actions[mod][button] = 
+				STATUS_DOCKLET_BUTTON_ACTION_NONE;
 			     
-	status_docklet_config.playing_image = g_strdup(XMMS_DATA_DIR "/status_docklet/stave-anim.xpm");
-	status_docklet_config.paused_image = g_strdup(XMMS_DATA_DIR "/status_docklet/rest.xpm");
+	status_docklet_config.playing_image = 
+		g_strdup(XMMS_DATA_DIR "/status_docklet/stave-anim.xpm");
+	status_docklet_config.paused_image = 
+		g_strdup(XMMS_DATA_DIR "/status_docklet/rest.xpm");
 	status_docklet_config.stopped_image = g_strdup("");
 
-	status_docklet_config.playing_image_delay = 100;
-	status_docklet_config.paused_image_delay = 100;
-	status_docklet_config.stopped_image_delay = 100;
+	status_docklet_config.playing_image_delay = 250;
+	status_docklet_config.paused_image_delay = 250;
+	status_docklet_config.stopped_image_delay = 250;
+
+	status_docklet_config.freedesktop_docklet_support = TRUE;
+	status_docklet_config.balloon_delay = 2;
 
 	cfgfile = xmms_cfg_open_default_file();
 	if (cfgfile) {
@@ -163,40 +204,67 @@ void status_docklet_load_config(void)
 						      (mod==STATUS_DOCKLET_BUTTON_MODIFIER_NONE) ? "" : button_modifier_text[mod],
 						      (mod==STATUS_DOCKLET_BUTTON_MODIFIER_NONE) ? "" : "_",
 						      button+1);
-				xmms_cfg_read_int(cfgfile, "status_docklet", key, (gint*)&status_docklet_config.button_actions[mod][button]);
+				xmms_cfg_read_int(cfgfile, STATUS_DOCKLET_CONFIG_SECTION, 
+						  key, (gint*)&status_docklet_config.button_actions[mod][button]);
 				g_free(key);
 			}
 		}
 	
-		if ( xmms_cfg_read_string (cfgfile, "status_docklet", "playing_image", &tmp) ) {
+		if ( xmms_cfg_read_string (cfgfile, STATUS_DOCKLET_CONFIG_SECTION, "playing_image", &tmp) ) {
 			g_free (status_docklet_config.playing_image);
 			status_docklet_config.playing_image = tmp;
 			tmp = NULL;
 		}
-		if ( xmms_cfg_read_string (cfgfile, "status_docklet", "paused_image", &tmp) ) {
+		if ( xmms_cfg_read_string (cfgfile, STATUS_DOCKLET_CONFIG_SECTION, "paused_image", &tmp) ) {
 			g_free (status_docklet_config.paused_image);
 			status_docklet_config.paused_image = tmp;
 			tmp = NULL;
 		}
 
-		if ( xmms_cfg_read_string (cfgfile, "status_docklet", "stopped_image", &tmp) ) {
+		if ( xmms_cfg_read_string (cfgfile, STATUS_DOCKLET_CONFIG_SECTION, "stopped_image", &tmp) ) {
 			g_free (status_docklet_config.stopped_image);
 			status_docklet_config.stopped_image = tmp;
 			tmp = NULL;
 		}
 
-		xmms_cfg_read_int (cfgfile, "status_docklet", "playing_image_delay", 
-				   &status_docklet_config.playing_image_delay);
-		xmms_cfg_read_int (cfgfile, "status_docklet", "paused_image_delay", 
-				   &status_docklet_config.paused_image_delay);
-		xmms_cfg_read_int (cfgfile, "status_docklet", "paused_image_delay", 
-				   &status_docklet_config.stopped_image_delay);
+		if ( xmms_cfg_read_int (cfgfile, STATUS_DOCKLET_CONFIG_SECTION, "playing_image_delay", 
+					&delay) &&
+		     delay > STATUS_DOCKLET_IMAGE_DELAY_MIN && 
+		     delay < STATUS_DOCKLET_IMAGE_DELAY_MAX )
+			status_docklet_config.playing_image_delay = delay;
+
+		if ( xmms_cfg_read_int (cfgfile, STATUS_DOCKLET_CONFIG_SECTION, "paused_image_delay", 
+					&delay) &&
+		     delay > STATUS_DOCKLET_IMAGE_DELAY_MIN && 
+		     delay < STATUS_DOCKLET_IMAGE_DELAY_MAX )
+			status_docklet_config.paused_image_delay = delay;
+
+		if ( xmms_cfg_read_int (cfgfile, STATUS_DOCKLET_CONFIG_SECTION, "paused_image_delay", 
+				   &delay) &&
+		     delay > STATUS_DOCKLET_IMAGE_DELAY_MIN && 
+		     delay < STATUS_DOCKLET_IMAGE_DELAY_MAX )
+			status_docklet_config.stopped_image_delay = delay;
+		
+		if ( xmms_cfg_read_int(cfgfile, STATUS_DOCKLET_CONFIG_SECTION, "balloon_delay", 
+				  &delay) &&
+		     delay > STATUS_DOCKLET_BALLOON_DELAY_MIN && 
+		     delay < STATUS_DOCKLET_BALLOON_DELAY_MAX )
+			status_docklet_config.balloon_delay = delay;
 
 		xmms_cfg_free(cfgfile);
 	}
 
 
 	status_docklet_load_images();
+
+	status_docklet_config.configured = TRUE;
+}
+
+static void status_docklet_freedesktop_support_toggled(GtkWidget *toggle, GtkWidget *spin)
+{
+	gboolean active = gtk_toggle_button_get_active
+		(GTK_TOGGLE_BUTTON(toggle));
+	gtk_widget_set_sensitive( spin, active );
 }
 
 static void status_docklet_cancel_cb(GtkWidget *widget, gpointer data)
@@ -214,9 +282,9 @@ static void status_docklet_create_button_pane(GtkWidget *notebook, StatusDocklet
 {
 	GtkWidget *table;
 	GtkWidget *option, *menu_item, *label;
-	gint i, j;
-
-	table = gtk_table_new ( 2, STATUS_DOCKLET_BUTTON_LAST, FALSE );	
+	gint button, action;
+	
+	table = gtk_table_new ( STATUS_DOCKLET_BUTTON_LAST, 2, FALSE );
 	gtk_container_set_border_width(GTK_CONTAINER(table), 5);
 	gtk_table_set_row_spacings(GTK_TABLE(table), 5);
 	gtk_table_set_col_spacings(GTK_TABLE(table), 2);
@@ -224,88 +292,41 @@ static void status_docklet_create_button_pane(GtkWidget *notebook, StatusDocklet
 
 	/* ------------------------- */
 
-	for ( i = 0; i < STATUS_DOCKLET_BUTTON_LAST; i++ ) {
-		button_action[modifier][i] = gtk_menu_new ();
-		for ( j = 0 ; j < STATUS_DOCKLET_BUTTON_ACTION_LAST ; j++ ) {
-			menu_item = gtk_menu_item_new_with_label ( _( button_action_text[j] ) );	
+	for ( button = 0; button < STATUS_DOCKLET_BUTTON_LAST; button++ ) {
+		button_action[modifier][button] = gtk_menu_new ();
+		for ( action = 0 ; action < STATUS_DOCKLET_BUTTON_ACTION_LAST ; action++ ) {
+			menu_item = gtk_menu_item_new_with_label ( _( button_action_text[action] ) );	
 				
-			gtk_object_set_data(GTK_OBJECT(menu_item), "cmd", GINT_TO_POINTER(j));
+			gtk_object_set_data(GTK_OBJECT(menu_item), "cmd", GINT_TO_POINTER(action));
 			gtk_object_set_data(GTK_OBJECT(menu_item), "mod", GINT_TO_POINTER(modifier));
 			gtk_widget_show(menu_item);
-			gtk_menu_append(GTK_MENU(button_action[modifier][i]), menu_item);
+			gtk_menu_append(GTK_MENU(button_action[modifier][button]), menu_item);
 		}
-		gtk_widget_show(button_action[modifier][i]);
+		gtk_widget_show(button_action[modifier][button]);
 	}
 
-	/* ------------------------- */
+	for(button = 0; button < STATUS_DOCKLET_BUTTON_LAST; button++ ) {
+		gchar *lbl;
+		lbl = g_strdup_printf(_ ("Button %d"), button+1);
+		label = gtk_label_new ( lbl );
+		g_free(lbl);
 
-	label = gtk_label_new (_ ("Button 1") );
-	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, GTK_SHRINK, GTK_SHRINK, 3, 3);
-	gtk_widget_show(label);
+		gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
 
-	option = gtk_option_menu_new();
-	gtk_option_menu_set_menu (GTK_OPTION_MENU(option), button_action[modifier][0]);
-	gtk_option_menu_set_history(GTK_OPTION_MENU(option), status_docklet_config.button_actions[modifier][0]);
-
-	gtk_table_attach(GTK_TABLE(table), option, 1, 2, 0, 1, GTK_EXPAND|GTK_FILL, GTK_SHRINK, 3, 3);
-	gtk_widget_show(option);
-
-	/* ------------------------- */
-
-	label = gtk_label_new (_ ("Button 2") );
-	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2, GTK_SHRINK, GTK_SHRINK, 3, 3);
-	gtk_widget_show(label);
-
-	option = gtk_option_menu_new();
-	gtk_option_menu_set_menu (GTK_OPTION_MENU(option), button_action[modifier][1]);
-	gtk_option_menu_set_history(GTK_OPTION_MENU(option), status_docklet_config.button_actions[modifier][1]);
-
-	gtk_table_attach(GTK_TABLE(table), option, 1, 2, 1, 2, GTK_EXPAND|GTK_FILL, GTK_SHRINK, 3, 3);
-	gtk_widget_show(option);
-
-	/* ------------------------- */
-
-	label = gtk_label_new (_ ("Button 3") );
-	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3, GTK_SHRINK, GTK_SHRINK, 3, 3);
-	gtk_widget_show(label);
-
-	option = gtk_option_menu_new();
-	gtk_option_menu_set_menu (GTK_OPTION_MENU(option), button_action[modifier][2]);
-	gtk_option_menu_set_history(GTK_OPTION_MENU(option), status_docklet_config.button_actions[modifier][2]);
-
-	gtk_table_attach(GTK_TABLE(table), option, 1, 2, 2, 3, GTK_EXPAND|GTK_FILL, GTK_SHRINK, 3, 3);
-	gtk_widget_show(option);
-
-	/* ------------------------- */
-
-	label = gtk_label_new (_ ("Button 4") );
-	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 3, 4, GTK_SHRINK, GTK_SHRINK, 3, 3);
-	gtk_widget_show(label);
-
-	option = gtk_option_menu_new();
-	gtk_option_menu_set_menu (GTK_OPTION_MENU(option), button_action[modifier][3]);
-	gtk_option_menu_set_history(GTK_OPTION_MENU(option), status_docklet_config.button_actions[modifier][3]);
-
-	gtk_table_attach(GTK_TABLE(table), option, 1, 2, 3, 4, GTK_EXPAND|GTK_FILL, GTK_SHRINK, 3, 3);
-	gtk_widget_show(option);
-
-	/* ------------------------- */
-
-	label = gtk_label_new (_ ("Button 5") );
-	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 4, 5, GTK_SHRINK, GTK_SHRINK, 3, 3);
-	gtk_widget_show(label);
-
-	option = gtk_option_menu_new();
-	gtk_option_menu_set_menu (GTK_OPTION_MENU(option), button_action[modifier][4]);
-	gtk_option_menu_set_history(GTK_OPTION_MENU(option), status_docklet_config.button_actions[modifier][4]);
-
-	gtk_table_attach(GTK_TABLE(table), option, 1, 2, 4, 5, GTK_EXPAND|GTK_FILL, GTK_SHRINK, 3, 3);
-	gtk_widget_show(option);
+		gtk_table_attach(GTK_TABLE(table), label, 0, 1, 
+				 button, button+1, 
+				 GTK_SHRINK, GTK_SHRINK, 3, 3);
+		gtk_widget_show(label);
+		
+		option = gtk_option_menu_new();
+		gtk_option_menu_set_menu (GTK_OPTION_MENU(option), 
+					  button_action[modifier][button]);
+		gtk_option_menu_set_history(GTK_OPTION_MENU(option), 
+					    status_docklet_config.button_actions[modifier][button]);
+		
+		gtk_table_attach(GTK_TABLE(table), option, 1, 2, button, button+1, GTK_EXPAND|GTK_FILL, GTK_SHRINK, 3, 3);
+		gtk_widget_show(option);
+	}
 
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), table, gtk_label_new( _( button_modifier_text[modifier] ) ) );
 }
@@ -313,6 +334,9 @@ static void status_docklet_create_button_pane(GtkWidget *notebook, StatusDocklet
 void status_docklet_configure(void)
 {
 
+	if ( !status_docklet_config.configured )
+		status_docklet_load_config();
+
 	if ( !config_window ) {
 		GtkWidget *vbox, *notebook, *box, *frame, *label;
 		GtkWidget *button_notebook, *button, *spin, *sep;
@@ -333,7 +357,7 @@ void status_docklet_configure(void)
 		gtk_box_pack_start ( GTK_BOX(vbox), notebook, TRUE, TRUE, 0);
 		gtk_widget_show(notebook);
 
-		frame = gtk_frame_new ( _("Button Actions") );
+		frame = gtk_frame_new ( NULL );
 		gtk_widget_show(frame);
 
 		button_notebook = gtk_notebook_new ();
@@ -348,7 +372,7 @@ void status_docklet_configure(void)
 		/* ------------------------- */
 		/* Images Page */
 
-		frame = gtk_frame_new ( _("Images") );
+		frame = gtk_frame_new ( NULL );
 		gtk_widget_show(frame);
 
 		table = gtk_table_new ( 0, 0, FALSE );	
@@ -387,7 +411,9 @@ void status_docklet_configure(void)
 		gtk_widget_show (label);
 
 		playing_image_adjustment = gtk_adjustment_new(status_docklet_config.playing_image_delay, 
-							      0.0, 1000.0, 10.0, 100.0, 100.0);
+							      STATUS_DOCKLET_IMAGE_DELAY_MIN, 
+							      STATUS_DOCKLET_IMAGE_DELAY_MAX, 
+							      10.0, 100.0, 100.0);
 		spin = gtk_spin_button_new( GTK_ADJUSTMENT(playing_image_adjustment), 10, 0);
 		gtk_table_attach ( GTK_TABLE(table), spin, 1, 2, 2, 3, GTK_EXPAND|GTK_FILL, GTK_SHRINK, 0, 1);
 		gtk_widget_show(spin);
@@ -425,7 +451,9 @@ void status_docklet_configure(void)
 		gtk_widget_show (label);
 
 		paused_image_adjustment = gtk_adjustment_new(status_docklet_config.paused_image_delay, 
-							     0.0, 1000.0, 10.0, 100.0, 100.0);
+							     STATUS_DOCKLET_IMAGE_DELAY_MIN, 
+							     STATUS_DOCKLET_IMAGE_DELAY_MAX, 
+							     10.0, 100.0, 100.0);
 		spin = gtk_spin_button_new( GTK_ADJUSTMENT(paused_image_adjustment), 10, 0);
 		gtk_table_attach ( GTK_TABLE(table), spin, 1, 2, 7, 8, GTK_EXPAND|GTK_FILL, GTK_SHRINK, 0, 1);
 		gtk_widget_show(spin);
@@ -462,7 +490,10 @@ void status_docklet_configure(void)
 		gtk_table_attach ( GTK_TABLE(table), label, 0, 1, 12, 13, GTK_SHRINK|GTK_FILL, GTK_SHRINK, 2, 1);
 		gtk_widget_show (label);
 
-		stopped_image_adjustment = gtk_adjustment_new(status_docklet_config.stopped_image_delay, 0.0, 1000.0, 10.0, 100.0, 100.0);
+		stopped_image_adjustment = gtk_adjustment_new(status_docklet_config.stopped_image_delay, 
+							      STATUS_DOCKLET_IMAGE_DELAY_MIN, 
+							      STATUS_DOCKLET_IMAGE_DELAY_MAX, 
+							      10.0, 100.0, 100.0);
 		spin = gtk_spin_button_new( GTK_ADJUSTMENT(stopped_image_adjustment), 10, 0);
 		gtk_table_attach ( GTK_TABLE(table), spin, 1, 2, 12, 13, GTK_EXPAND|GTK_FILL, GTK_SHRINK, 0, 1);
 		gtk_widget_show(spin);
@@ -471,6 +502,50 @@ void status_docklet_configure(void)
 
 		gtk_notebook_append_page(GTK_NOTEBOOK(notebook), frame, gtk_label_new(_("Docklet Images")));
 
+		/* ------------------------- */
+		/* Freedesktop.org Support Page */
+
+		frame = gtk_frame_new ( NULL );
+		gtk_widget_show(frame);
+		
+		table = gtk_table_new ( 0, 0, FALSE );	
+		gtk_container_set_border_width(GTK_CONTAINER(table), 5);
+		gtk_container_add ( GTK_CONTAINER(frame), table );
+		gtk_table_set_row_spacings(GTK_TABLE(table), 5);
+		gtk_table_set_col_spacings(GTK_TABLE(table), 5);
+		gtk_widget_show(table);
+
+		freedesktop_support_check_button 
+			= gtk_check_button_new_with_label(_("Support freedesktop.org System Tray\n" 
+							    "Specification. (Requires restart)"));
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(freedesktop_support_check_button),
+					     status_docklet_config.freedesktop_docklet_support);
+
+		gtk_table_attach ( GTK_TABLE(table), freedesktop_support_check_button, 0, 2, 0, 1, 
+				   GTK_SHRINK|GTK_FILL, GTK_SHRINK, 2, 1);
+		gtk_widget_show(freedesktop_support_check_button);
+
+		label = gtk_label_new ( _("Balloon Timeout (seconds)") );
+		gtk_misc_set_alignment ( GTK_MISC (label), 0.0, 0.5);
+		gtk_table_attach ( GTK_TABLE(table), label, 0, 1, 1, 2, GTK_SHRINK|GTK_FILL, GTK_SHRINK, 2, 1);
+		gtk_widget_show (label);
+
+		balloon_delay_adjustment = gtk_adjustment_new(status_docklet_config.balloon_delay, 
+							      STATUS_DOCKLET_BALLOON_DELAY_MIN, 
+							      STATUS_DOCKLET_BALLOON_DELAY_MAX, 
+							      1.0, 1.0, 1.0);
+		spin = gtk_spin_button_new( GTK_ADJUSTMENT(balloon_delay_adjustment), 10, 0);
+		gtk_widget_set_sensitive( GTK_WIDGET(spin), status_docklet_config.freedesktop_docklet_support);
+		
+		gtk_signal_connect( GTK_OBJECT(freedesktop_support_check_button), "toggled",
+				    (GtkSignalFunc)status_docklet_freedesktop_support_toggled, spin);				    
+		gtk_table_attach ( GTK_TABLE(table), spin, 1, 2, 1, 2, GTK_EXPAND|GTK_FILL, GTK_SHRINK, 0, 1);
+		gtk_widget_show(spin);
+
+		/* ------------------------- */
+		
+		gtk_notebook_append_page(GTK_NOTEBOOK(notebook), frame, gtk_label_new(_("System Tray Protocol")));
+		
 		/* ------------------------- */
 		
 		box = gtk_hbutton_box_new();
